@@ -1,5 +1,5 @@
 # prime/engine/engine-basic.rb
-# $Id: engine-userdict.rb,v 1.11 2004/12/09 12:24:15 komatsu Exp $
+# $Id: engine-userdict.rb,v 1.9 2004/03/25 17:19:50 komatsu Exp $
 #
 # Copyright (C) 2002 Hiroyuki Komatsu <komatsu@taiyaki.org>
 #     All rights reserved.
@@ -150,7 +150,6 @@ class PrimeEngineUserdict < PrimeEngine
     unless $PRIME_NO_SAVE then
       Thread.new do
         system("#{PRIME_COMMAND_USERDICT_UPDATE} --auto --quiet")
-        initialize_userdict()
       end
     end
   end
@@ -260,7 +259,8 @@ class PrimeEngineUserdict < PrimeEngine
       (key, value) = item.split(/=/)
       data[key] = value
     }
-    word = PrimeWord.new(pron, literal, pos, score, data)
+    word = PrimeWord.new(pron, literal, pos, score,
+                         data['category'], data['annotation'], data_list)
   end
 
   def check_existence (pron, literal, pos)
