@@ -1,5 +1,5 @@
 # prime/prime-japnese.rb: Japanese module for PRIME.
-# $Id: prime-japanese.rb,v 1.10 2004/12/18 08:02:53 komatsu Exp $
+# $Id: prime-japanese.rb,v 1.4.4.1 2004/06/01 06:41:28 komatsu Exp $
 #
 # Copyright (C) 2004 Hiroyuki Komatsu <komatsu@taiyaki.org>
 #     All rights reserved.
@@ -36,11 +36,10 @@ module PrimeJapanese
       'ư::' => ['֤::', 'Ǿá֤ʤ::'],
       'ư' => ['Ը', 'Ը', 'Ը', 'Ը', 'ʹԸ', 
 	'йԸ', '޹Ը', 'Ը', 'Ը', '(Ԥ)',
-	'', 'ư', '()', '&̾'],
+	'', 'ư', '()'],
       '' => ['ʽ֤ˡ', 'ʽ֤ǡ', 'ʽ֤',
-                 'ʽ֤ȡ', 
-	'ʽ֤Ρ', 'ʽ֤', '֤ϡ', '³֤ȡ'],
-      '̾' => ['̾', '()&̾', 'ư&̾', '&̾'],
+	'ʽ֤Ρ', 'ʽ֤', '֤ϡ'],
+      '̾' => ['̾', '()&̾', 'ư&̾'],
     }
 
     @pos_alias_reverse = {}
@@ -57,11 +56,11 @@ module PrimeJapanese
     @pos_connection_table = {
       '' => [''],
       'Ƭ' => ['̾'],
-      '̾'   => ['', 'ư첽', '̾', 'ƻ', 'ư'],
+      '̾'   => ['', 'ư첽', '̾'],
 
-      '̾&̾'   => ['̾', '', 'ư첽'],
+      '̾&̾'   => ['', 'ư첽'],
+      '̾'        => ['', 'ư첽'],
       '̾'        => ['', 'ư첽'],
-      '̾'        => ['̾', '', 'ư첽'],
 
       ## FIXME: ϡֽߡפǤϤʤϢΡ
       ## FIXME: <komatsu@taiyaki.org> (2004-01-19)
@@ -72,7 +71,7 @@ module PrimeJapanese
       'ư::Ϣi' => ['ư'],
       '' => ['ƻ', 'ư', 'ư&̾', 'ư'],
       'ƻ::Ϣ'   => ['̾', '̾&̾', '̾', '̾'],
-      'ƻ::Ϣku' => ['ƻ', 'ư'],
+      'ƻ::Ϣku' => ['ƻ'],
       'ư::Ϣ' => ['̾', '̾&̾', '̾', '̾'],
       'ư' => ['̾', '̾&̾', '̾', '̾'],
       '֤::Ϣde' => ['ư'],
@@ -90,8 +89,6 @@ module PrimeJapanese
 #      "֤::\t̾" => 0.9,
 #      "Ǿá֤ʤ::\t̾" => 0.9,
       "\t"  => 0.9,
-      "̾&̾\t̾" => 0.99,
-      "̾\t̾" => 0.99,
       "̾&̾\t=" => 0.99,
       "̾\t="      => 0.99,
       "=\t̾"   => 0.99,
@@ -102,14 +99,6 @@ module PrimeJapanese
       "=\t̾" => 0.99,
       "=\t̾" => 0.99,
       "=ɤ\t̾" => 0.99,
-      "=\t()&̾"   => 0.99,
-      "=\t()&̾"   => 0.99,
-      "=\t()&̾"   => 0.99,
-      "=ɤ\t()&̾"   => 0.99,
-      "=\t()&̾" => 0.99,
-      "=\t()&̾" => 0.99,
-      "=\t()&̾" => 0.99,
-      "=ɤ\t()&̾" => 0.99,
     }
     @pos_connection_cost_default = 0.8
     @pos_connection_pos = {
@@ -119,11 +108,9 @@ module PrimeJapanese
       "\t" => '̾',
       "̾\t" => '̾',
       "̾\tư첽" => 'ư',
-      "̾&̾\t̾" => '̾',
-      "̾&̾\t"   => '̾',
+      "̾&̾\t" => '̾',
       "̾&̾\tư첽" => 'ư',
-      "̾\t̾" => '̾',
-      "̾\t"   => '̾',
+      "̾\t" => '̾',
       "̾\tư첽" => 'ư',
       "̾\t" => '̾',
       "̾\tư첽" => 'ư',
@@ -155,8 +142,12 @@ module PrimeJapanese
 
       connection_key = [pos, pos1].join("\t")
       cost = _get_connection_cost(pos, literal, pos1, literal1)
+
+      ## FIXME: How far is a range of glue.
+      ## FIXME: <komatsu@taiyaki.org> (2004-05-13)
+      glue = Prime::get_prefix(literal + conjugation, literal1)
       pron    += (conjugation + pron1)
-      literal += (conjugation + literal1)
+      literal += (conjugation + glue + literal1)
       score    = (Math::sqrt(score * score1) * cost).to_i
       pos = (@pos_connection_pos[connection_key] or pos1)
       conjugation = word1.conjugation
@@ -231,31 +222,7 @@ module PrimeJapanese
           end
           results.push(word)
         end
-      end
-    }
-
-    return results
-  end
-
-  def search_japanese_uniclause (string)
-    string = PrimeTypeConv::convert(string).join
-
-    results = search_raw(string)
-    rest = nil
-
-    clauses_list = process_clauses(string, 1)
-    clauses_list.each {|clauses|
-      if clauses.length == 1 then
-        ## FIXME: It's an adhoc routine.
-        ## FIXME: <komatsu@taiyaki.org> (2004-01-26)
-        (pron, pos, conjugation, conjugation_pos) = clauses.first
-        query = PrimeQuery.new([pron], pos)
-        words = search(query)
-        words.each {|word|
-          word.conjugation = conjugation
-        }
-        results += words
-      end
+       end
     }
 
     return results
@@ -282,8 +249,8 @@ module PrimeJapanese
   ## FIXME: ʸڤ꤬ƱΤ礷
   ## FIXME: <komatsu@taiyaki.org> (2004-01-24)
 
-  def process_clauses (string, threshold = 4)
-    (depth, clauses_list) = guess_clauses_internal(string, 1, nil, threshold)
+  def process_clauses (string)
+    (depth, clauses_list) = guess_clauses_internal(string)
     processed_clauses_list = []
     clauses_list.each {|clauses|
       if clauses.length <= depth then
@@ -387,7 +354,7 @@ class PrimeTypeConv
         PRIME_ENV['typing_method'] == nil then
       if conversion =~ /[a-zA-Z]/ then
         return [string]
-      elsif string =~ /^-?[0-9+][0-9,.+-]*/ then
+      elsif string =~ /^[0-9+-][0-9,.+-]*/ then
         numeral = $&
         string = string[numeral.length..-1]
         (conversion, pending, node) = suikyo.convert_internal(string)
@@ -417,41 +384,24 @@ class PrimeTypeConv
       return @@suikyo
     end
 
-    ## lock for "suikyo.rbo"
-    filename_lock = File::join2(PRIME_USER_DIR, ".lock")
-    if File::symlink?(filename_lock) == false or
-      ( File::symlink?(filename_lock) and
-          Time::now - File::lstat(filename_lock).mtime > 3600 ) then
-
-      unless File::symlink?(filename_lock) then
-        File::symlink($$.to_s, filename_lock)
-      end
-
-      if PRIME_ENV['suikyo_use_cache'] and force == false then
-        threshold_time = [File::mtime(File::expand_path(__FILE__)),
-          File::mtime(PRIME_CONFIGFILE_GLOBAL),
-          File::mtime(PRIME_CONFIGFILE_LOCAL)].max
-      else
-        threshold_time = Time.new()
-      end
-      filename_cache = File::join2(PRIME_USER_DIR, "suikyo.rbo")
-
-      suikyo = Marshal::init_file(filename_cache, threshold_time) {
-        PrimeTypeConv::initialize_suikyo_internal(typing_method)
-      }
-
-      suikyo.table.table_files.each {|file|
-        if File::mtime(filename_cache) < File::mtime(file) then
-          return PrimeTypeConv::initialize_suikyo(true)
-        end
-      }
-
-      ## unlock for "suikyo.rbo"
-      File::delete(filename_lock)
+    if PRIME_ENV['suikyo_use_cache'] and force == false then
+      threshold_time = [File::mtime(File::expand_path(__FILE__)),
+                        File::mtime(PRIME_CONFIGFILE_GLOBAL),
+                        File::mtime(PRIME_CONFIGFILE_LOCAL)].max
     else
-      ## If the locking failed, the cache file is not used.
-      suikyo = PrimeTypeConv::initialize_suikyo_internal(typing_method)
+      threshold_time = Time.new()
     end
+    filename_cache = File::join2(PRIME_USER_DIR, "suikyo.rbo")
+    suikyo = Marshal::init_file(filename_cache, threshold_time) {
+      suikyo = Suikyo.new
+      tables = (PRIME_ENV['suikyo_tables'] or
+                  PRIME_TYPING_TABLE[typing_method] or
+                  [])
+      tables.each {|table|
+	suikyo.table.loadfile(table)
+      }
+      suikyo
+    }
 
     # Version checking
     if suikyo.respond_to?('convert_internal') == false then
@@ -464,6 +414,12 @@ class PrimeTypeConv
       exit()
     end
 
+    suikyo.table.table_files.each {|file|
+      if File::mtime(filename_cache) < File::mtime(file) then
+        return PrimeTypeConv::initialize_suikyo(true)
+      end
+    }
+
     PRIME_ENV['suikyo_tables'] = suikyo.table.table_files
     if PRIME_ENV['suikyo_reverse_tables'].nil? then
       PRIME_ENV['suikyo_reverse_tables'] = 
@@ -488,16 +444,4 @@ class PrimeTypeConv
     @@suikyo = suikyo
     return suikyo
   end
-
-  ## This initializes the instance of Suikyo.
-  def PrimeTypeConv::initialize_suikyo_internal (typing_method)
-    suikyo = Suikyo.new()
-    tables = (PRIME_ENV['suikyo_tables'] or
-                PRIME_TYPING_TABLE[typing_method] or
-                [])
-    tables.each {|table|
-      suikyo.table.loadfile(table)
-    }
-    return suikyo
-  end
 end
