@@ -1,5 +1,5 @@
 # prime/prime.rb
-# $Id: prime.rb,v 1.17 2004/12/31 06:23:32 komatsu Exp $
+# $Id: prime.rb,v 1.7.4.4 2004/06/06 06:31:25 komatsu Exp $
 #
 # Copyright (C) 2002, 2003, 2004 Hiroyuki Komatsu <komatsu@taiyaki.org>
 #     All rights reserved.
@@ -56,7 +56,7 @@ class Prime
     case PRIME_ENV['typing_method']
     when 'tcode' then
       if PRIME_ENV.has_key?('style_mask_pending_chars') == false then
-        PRIME_ENV['style_mask_pending_chars'] = true
+        PRIME_ENV['style_mask_pending_chars'] == true
       end
     end
   end
@@ -121,7 +121,7 @@ class Prime
   end
 
   def preedit_convert_input (pattern)
-    return PrimeTypeConv::convert(pattern).join("\t")
+    return PrimeTypeConv::convert(pattern).join()
   end
 
   def set_context (context)
@@ -145,7 +145,8 @@ class Prime
         PRIME_ENV['typing_method'] == 'handwrite' then
       word_list = lookup_direct(string)
     else
-      word_list = lookup_compact(string)
+#      word_list = lookup_compact(string)
+      word_list = lookup_hybrid(string)
     end
     return word_list
   end
@@ -154,21 +155,21 @@ class Prime
         PRIME_ENV['typing_method'] == 'handwrite' then
       word_list = lookup_direct_all(string)
     else
-      word_list = lookup_compact_all(string)
+      word_list = lookup_hybrid_all(string)
     end
     return word_list
   end
 
   def lookup_hybrid (string)
-    if @cache_hybrid_pattern == string then
-      return @cache_hybrid
-    end
+#     if @cache_hybrid_pattern == string then
+#       return @cache_hybrid
+#     end
 
     words  = search_hybrid(string, 10)
     result = PrimeWordList::merge_with_label(@context, words)
 
-    @cache_hybrid_pattern = string
-    @cache_hybrid = result
+#     @cache_hybrid_pattern = string
+#     @cache_hybrid = result
     return result
   end
   def lookup_hybrid_all (string)
@@ -179,7 +180,7 @@ class Prime
 
   def lookup_direct (string)
     results_expansion = lookup_expansion(string)
-    results = PrimeWordList::merge_by_literal(@context, results_expansion)
+    results = PrimeWordList::merge_with_label(@context, results_expansion)
     return results[0,3]
   end
 
@@ -190,7 +191,7 @@ class Prime
     words_japanese  = search_japanese(string)
     words_overall   = search_overall(string)
 
-    results = PrimeWordList::merge_by_literal(@context,
+    results = PrimeWordList::merge_with_label(@context,
                                               # results_prefix[0, 3],
                                               results_expansion,
                                               results_mixed,
@@ -201,18 +202,19 @@ class Prime
 
   def lookup_compact (string)
     words_compact = search_compact(string)
-    words = PrimeWordList::attach_prefix(@context, words_compact)
+    words = PrimeWordList::merge_with_label(@context, words_compact)
     return words
   end
   def lookup_compact_all (string)
-    words_compact  = lookup_compact(string)
+    words_compact  = search_compact(string)
     words_overall  = search_overall(string)
     words_japanese = search_japanese(string)
-    
+    results_compact    = PrimeWordList::merge_with_label(@context,
+                                                         words_compact)
     results_conversion = PrimeWordList::merge_with_label(@context,
                                                          words_overall,
                                                          words_japanese)
-    return PrimeWordList::concat(words_compact | results_conversion)
+    return PrimeWordList.new(results_compact | results_conversion)
   end
 
   def lookup_prefix (string)
@@ -220,14 +222,6 @@ class Prime
     return PrimeWordList::merge_with_label(@context, words_prefix)
   end
 
-  def lookup_prefix_ex (string)
-    words_prefix      = search_prefix(string)
-    results_expansion = lookup_expansion(string)
-    return PrimeWordList::merge_with_label(@context,
-					   words_prefix,
-					   results_expansion)
-  end
-
   def lookup_exact (string)
     words_exact = search_exact(string)
     return PrimeWordList::merge_with_label(@context, words_exact)
@@ -262,17 +256,17 @@ class Prime
   end
 
   def search_prefix (string)
-    if @cache_prefix_pattern == string then
-      return @cache_prefix
-    end
+#     if @cache_prefix_pattern == string then
+#       return @cache_prefix
+#     end
 
     # ֤袪ͽ¬
     expands = PrimeTypeConv::expand(string)
     query = PrimeQuery.new(expands, nil, :prefix, @context)
     words = search(query)
 
-    @cache_prefix_pattern = string
-    @cache_prefix = words
+#     @cache_prefix_pattern = string
+#     @cache_prefix = words
 
     return words
   end
@@ -300,28 +294,12 @@ class Prime
   end
 
   def search_compact (string)
-    if @cache_compact_pattern == string then
-      return @cache_compact
-    end
-    words_prefix    = search_prefix(string)
-    words = PrimeWordList::merge_with_label(@context, 
-                                            words_prefix[0,1])
-
-#     words_uniclause = search_japanese_uniclause(string)
-
-#     words = PrimeWordList::merge_with_label(@context, 
-#                                             words_prefix[0,1],
-#                                             words_uniclause)
-
-
-    ## Ruby 1.6 does not keep the class PrimeWordList word[0,1] if the
-    ## value of word is [], and the class of the result of word[0,1]
-    ## becomes Array which is a super class of PrimewordList.
-    if words.empty? then
-      words_compact = PrimeWordList.new()
-    else      
-      words_compact = words[0,1]
-    end
+#     if @cache_compact_pattern == string then
+#       return @cache_compact
+#     end
+    expands = PrimeTypeConv::expand(string)
+    query   = PrimeQuery.new(expands, nil, :prefix, @context)
+    words_compact = search(query)[0,1]
 
     if words_compact.length > 0 then
       predict_with_multi_clauses!(words_compact)
@@ -331,8 +309,8 @@ class Prime
       }
     end
 
-    @cache_compact_pattern = string
-    @cache_compact         = words_compact
+#     @cache_compact_pattern = string
+#     @cache_compact         = words_compact
     return words_compact
   end
 
@@ -380,8 +358,6 @@ class Prime
   end
 
   public
-  ## This returns an avairable prefix string for the literal.
-  ## ex). get_prefix("This", "is") => " "
   def Prime::get_prefix (context, literal)
     if PRIME_ENV['style_auto_space'] and context and context != "" then
       if (context[-1] > 128 and literal[0] > 128) or
@@ -410,17 +386,18 @@ class PrimeQuery
   end
 end
 
-## PrimeWord contains data of a word generated by a user's query.
 class PrimeWord
   attr_reader   :pron, :literal, :pos, :index
-  attr_accessor :data, :score, 
+  attr_accessor :category, :annotation, :data, :score, 
       :conjugation, :conjugation_pos, :rest, :prefix
-  def initialize(pron, literal, pos, score, data = {})
-
+  def initialize(pron, literal, pos, score,
+                 category = nil, annotation = nil, *data)
     @pron        = (pron    or "")
     @literal     = (literal or "")
     @pos         = (pos     or "̤θ")
     @score       = score.to_i
+    @category    = category
+    @annotation  = annotation
     @data        = data
     @conjugation = ""
     @rest        = ""
@@ -430,7 +407,7 @@ class PrimeWord
   end
 
   def values
-    return [@pron, @literal, @pos, @score, @data]
+    return [@pron, @literal, @pos, @score, @category, @annotation, @data]
   end
 
   def label ()
@@ -444,29 +421,28 @@ class PrimeWord
     return (@pron + @conjugation + @rest)
   end
 
-  ## This returns a string data of this word for sending to clients.
   def to_text
-    data_list = [
-      to_text_pron(),
-      to_text_literal(),
-      "priority=#{@score}",
-      "part=#{@pos}",
-      "base=#{@literal}",
-      "basekey=#{@pron}",
-      ( "conjugation=#{@conjugation}"  unless @conjugation.empty?() ),
-      ( "suffix=#{@rest}"              unless @rest.empty?()        ),
-      ( "form=#{@data['form']}"        if @data.has_key?('form')    ),
-      ( "usage=#{@data['usage']}"      if @data.has_key?('usage')   ),
-      ( "comment=#{@data['comment']}"  if @data.has_key?('comment') ),
-    ]
-    return data_list.compact.join("\t")
+    return [to_text_pron(),
+            to_text_literal(),
+            "priority=#{@score}",
+            "part=#{@pos}",
+            "base=#{@literal}",
+            "basekey=#{@pron}",
+            ("conjugation=#{@conjugation}" unless @conjugation.empty?),
+            ("suffix=#{@rest}"             unless @rest.empty?)
+    ].compact.join("\t")
   end
 end
 
 class PrimeWordList < Array
-  def PrimeWordList::concat(*words)
-    ## This method does not sort the given words.
-    result = words.flatten.compact
+  def PrimeWordList::merge (*words)
+    ## merge should be sorted by inter-engine score.
+    ## FIXME: Isn't it necessary to sort here?
+    ## FIXME: <komatsu@taiyaki.org> (2004-01-24)
+    result = words.flatten.compact.sort {|word1, word2| 
+      (word2.score != word1.score) ? (word2.score <=> word1.score) :
+             (word1.pron.length <=> word2.pron.length)
+    }
 
     mark = {}
     merged = PrimeWordList.new
@@ -475,13 +451,8 @@ class PrimeWordList < Array
         word0 = mark[word.index]
         ## FIXME: Make this merging more intelligence.
         ## FIXME: <komatsu@taiyaki.org> (2004-01-24)
-        word.data.keys().each { | key |
-          if word0.data.has_key?(key) then
-            # Do nothing yet
-          else
-            word0.data[key] = word.data[key]
-          end
-        }
+        word0.category   = (word0.category   or word.category)
+        word0.annotation = (word0.annotation or word.annotation)
       else
 	mark[word.index] = word
 	merged.push(word)
@@ -490,56 +461,29 @@ class PrimeWordList < Array
     return merged
   end
 
-  def PrimeWordList::sort (*words)
-    result = words.flatten.compact.sort {|word1, word2| 
-      (word2.score != word1.score) ? (word2.score <=> word1.score) :
-                                     (word1.pron.length <=> word2.pron.length)
-    }
-    return result
-  end
-
-  def PrimeWordList::merge (*words)
-    return PrimeWordList::merge_internal(:index, nil, words)
-  end
-
   ## FIXME: Change the name of method.
   ## FIXME: <komatsu@taiyaki.org> (2004-01-26)
   def PrimeWordList::merge_with_label (context, *words)
-    return PrimeWordList::merge_internal(:label, context, words)
-  end
-
-  def PrimeWordList::merge_by_literal (context, *words)
-    return PrimeWordList::merge_internal(:literal, context, words)
-  end
-
-  def PrimeWordList::merge_internal (get_key_function, context, words)
     ## merge should be sorted by inter-engine score.
     ## FIXME: Isn't it necessary to sort here?
     ## FIXME: <komatsu@taiyaki.org> (2004-01-24)
     result = words.flatten.compact.sort {|word1, word2| 
       (word1.score != word2.score) ? (word2.score <=> word1.score) :
-        (word2.pron.length <=> word1.pron.length)
+             (word2.pron.length <=> word1.pron.length)
     }
 
     mark = {}
-    merged = PrimeWordList.new()
+    merged = PrimeWordList.new
     result.each {|word|
       if word.literal != "" then
-        word_key = word.send(get_key_function)
-        if mark[word_key] then
-          word0 = mark[word_key]
-
+        if mark[word.label()] then
+          word0 = mark[word.label()]
           ## FIXME: Make this merging more intelligence.
           ## FIXME: <komatsu@taiyaki.org> (2004-01-24)
-          word.data.keys().each { | key |
-            if word0.data.has_key?(key) then
-              # Do nothing yet
-            else
-              word0.data[key] = word.data[key]
-            end
-          }
+          word0.category   = (word0.category   or word.category)
+          word0.annotation = (word0.annotation or word.annotation)
         else
-          mark[word_key] = word
+          mark[word.label()] = word
           word.prefix = Prime::get_prefix(context, word.literal)
           merged.push(word)
         end
@@ -548,15 +492,6 @@ class PrimeWordList < Array
     return merged
   end
 
-  ## This attaches a prefix characters depend on the context,
-  ## and returns the given words.
-  def PrimeWordList::attach_prefix (context, words)
-    words.each { | word |
-      word.prefix = Prime::get_prefix(context, word.literal)
-    }
-    return words
-  end
-
   def to_text
     texts = self.map {|word|
       word.to_text
